------------------------------------------------------
-- Main.lua
------------------------------------------------------
local addon, _ = ...
local version = GetAddOnMetadata(addon, "Version");

-- make sure classic version...
local classicOnly = true;
local toc = select(4, GetBuildInfo());
if ( toc > 19999 ) then
	print("|cffa330c9"..addon.." "..version..": Classic Version Only!|r");
	classicOnly = false;
end;

local CALENDAR_FILTER_DARKMOON = _G.CALENDAR_FILTER_DARKMOON;
local ITEM_BIND_ON_PICKUP = _G.ITEM_BIND_ON_PICKUP;
local ITEM_BIND_TO_BNETACCOUNT = _G.ITEM_BIND_TO_BNETACCOUNT;
local ITEM_BIND_TO_ACCOUNT = _G.ITEM_BIND_TO_ACCOUNT;
local MERCHANT_ITEMS_PER_PAGE = _G.MERCHANT_ITEMS_PER_PAGE;
local UNKNOWN = _G.UNKNOWN;

FT_NewItemInfo = {};
FT_NewVendorInfo = {};
FT_NewVendorLocation = {};

local defaultsDB = {
	CoinFormat = "icon", --< icon, short, long
	NoDisplay = false,
	NoSound = false,
};

function initiateDB(a,b)
	if type(a) ~= "table" then return {}; end;
	if type(b) ~= "table" then b = {}; end;
	for k, v in pairs(a) do
		if type(v) == "table" then
			b[k] = initiateDB(v, b[k]);
		elseif type(b[k]) ~= type(v) then
			b[k] = v;
		end;
	end;
	return b;
end;

local FT_VersionFrame = CreateFrame("FRAME");
FT_VersionFrame:RegisterEvent("ADDON_LOADED");
function FT_VersionFrame:OnEvent(event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon then
		FleecingTipDB = initiateDB(defaultsDB, FleecingTipDB)
		if FleecingTipDB.Version == nil then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		elseif FleecingTipDB.Version ~= version then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		end;
		FT_VersionFrame:UnregisterEvent("ADDON_LOADED");
		if ( classicOnly ) then
			FTUtils.PrintOnce(addon.." "..FTUtils.LightYellow(version)..": addon Loaded.", 60);
		end;
	end;
end;
FT_VersionFrame:SetScript("OnEvent", FT_VersionFrame.OnEvent);

function FT_HookTooltip(frame)
	if (frame:GetScript("OnTooltipSetItem")) then
		frame:HookScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	else
		frame:SetScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	end;
end;

--[[ check active event at present time...
function CheckEventActive(worldEvent)
	if (not worldEvent) then return ""; end;
	
	-- Darkmoon Faire Only
	if (worldEvent == FT.DARKMOON_FAIRE) then
		local e;
		local date = C_Calendar.GetDate();
		local today = date.monthDay;
		local numDayEvents = C_Calendar.GetNumDayEvents(0,today);
		for e = 1,numDayEvents do
			local event = C_Calendar.GetDayEvent(0, today, e);
			if event.title == CALENDAR_FILTER_DARKMOON then
				return "|cff00ff00"..FT.ACTIVE.."|r";
			end;
		end;
		return "|cffff0000"..FT.NO_ACTIVE.."|r";
	end;
	
	
	local WEList = {
	-- Children's Week
--	[FT.HOLIDAY_EVENT_CW] = { 235444, 235445, 235446 },
	-- Brewfest
	[FT.HOLIDAY_EVENT_Brew] = { 0, 0, 0 },
	-- Hallow's End
	[FT.HOLIDAY_EVENT_HE] = { 235460, 235461, 235462 },
	-- Love is in the Air
	[FT.HOLIDAY_EVENT_LiitA] = { 0, 0, 0 },
	-- Lunar Festival
	[FT.HOLIDAY_EVENT_LF] = { 235469, 235470, 235471 },
	-- Harvest Festival
	[FT.HOLIDAY_EVENT_HF] = { 235472, 235473, 235474 },
	-- Noblegarden
	[FT.HOLIDAY_EVENT_Noble] = { 235475, 235476, 235477 },
	-- Midsummer Fire Festival
	[FT.HOLIDAY_EVENT_MFF] = { 0, 0, 0 },
	};
	
	local calendar = C_Calendar.GetDate();
	local month, day, year = calendar.month, calendar.monthDay, calendar.year;
	local monthInfo = C_Calendar.GetMonthInfo();
	local curMonth, curYear = monthInfo.month, monthInfo.year;
	local monthOffset = -12 * (curYear - year) + month - curMonth;
	local numEvents = C_Calendar.GetNumDayEvents(monthOffset, day);
--	print("numEvents = "..tostring(numEvents));
--	local event = C_Calendar.GetDayEvent(monthOffset, day, 1);
--	print("event.iconTexture = "..tostring(event.iconTexture));
--	print("event.sequenceType = "..tostring(event.sequenceType));
--	print("event.title = "..tostring(event.title));
--	local event2 = C_Calendar.GetHolidayInfo(monthOffset, day, 1);
--	print("event2.name = "..tostring(event2.name));
--	print("event2.texture = "..tostring(event2.texture));
	
	for i=1, numEvents do
		local event = C_Calendar.GetDayEvent(monthOffset, day, i);
		
--		print("event.iconTexture = "..tostring(event.iconTexture));
--		event.sequenceType = "ONGOING";
--		print("event.sequenceType = "..tostring(event.sequenceType));
		
		local listEvent = WEList[worldEvent];
--		listEvent = WEList[FT.HOLIDAY_EVENT_Noble];
		if (listEvent) then
--			print("YES!!!");
			local okay = nil;
			for w=1, table.getn(listEvent) do
--				print("listEvent[w] = "..tostring(listEvent[w]));
				local eventFound = event.iconTexture;
				if (listEvent[w] == eventFound) then okay = true; end;
			end;
			if (okay) then
--				print("OKAY!!!");
				if (event.sequenceType == "ONGOING") then
					return "|cff00ff00"..FT.ACTIVE.."|r";
				else
					local hour = GetGameTime();
					if event.sequenceType == "END" and hour <= event.endTime.hour or
						event.sequenceType == "START" and hour >= event.startTime.hour then
						return "|cff00ff00"..FT.ACTIVE.."|r";
					end;
				end;
			else
				return "|cffff0000"..FT.NO_ACTIVE.."|r";
			end;
		else	
			return "|cffff0000"..UNKNOWN.."|r";
		end;

	end;
	return "---";
end;
]]--
-- return the real zone name for BC or WoD...
function FT_RealMapName(mapID)
	if (not mapID) then return nil; end;
	-- some reason no mapID for instances!
	local ClassicDungeons = {
		[33] = "Shadowfang Keep",
		[34] = "The Stockade",
		[36] = "The Deadmines",
		[43] = "Wailing Caverns",
		[47] = "Razorfen Kraul",
		[48] = "Blackfathom Deeps",
		[70] = "Uldaman",
		[90] = "Gnomeregan",
		[109] = "The Temple of Atal'Hakkar",
		[129] = "Razorfen Downs",
		[209] = "Zul'Farrak",
		[229] = "Blackrock Spire",
		[230] = "Blackrock Depths",
		[249] = "Onyxia's Lair",
		[289] = "Scholomance",
		[309] = "Zul'Gurub",
		[329] = "Stratholme",
		[349] = "Maraudon",
		[389] = "Ragefire Chasm",
		[407] = "Darkmoon Faire",
		[409] = "Molten Core",
		[429] = "Dire Maul",
		[469] = "Blackwing Lair",
		[509] = "Ruins of Ahn'Qiraj",
		[531] = "Temple of Ahn'Qiraj",
		[533] = "Naxxramas",
		[1001] = "Scarlet Halls",
		[1004] = "Scarlet Monastery",
		[1007] = "Scholomance",
	};
	
	local mapName = ClassicDungeons[mapID];
	if (not mapName) then
		local mapDetails = C_Map.GetMapInfo(mapID);
		mapName = mapDetails.name;
	end;
	return mapName;
end;

function FT_GetVendorCoordinates(faction,npc)
	if (not faction) or (not npc) then return nil; end;
	local tex = nil;
	local vendorCoords = FT_VendorLocations[faction][npc];
--	print("faction, npc, vendorCoords = "..tostring(faction)..", "..tostring(npc)..", "..tostring(vendorCoords[1]));
	if ( vendorCoords ~= nil) then
		local uiD = vendorCoords[1];
		if (not uiD) then return nil; end;
		local a1,c1 = "", "";
		if (uiD > 0) then
			a1 = FT_RealMapName(uiD);
		end;
		local b1 = FT_VendorLocations[faction][npc][2] or "";
		if (string.len(b1) == 4) then b1 = string.format("%.1f|r",tonumber(b1)); end;
		local d1 = FT_VendorLocations[faction][npc][3] or "";
		if (string.len(d1) == 4) then d1 = string.format("%.1f|r",tonumber(d1)); end;
		if (FT_VendorLocations[faction][npc][3]) then c1 = ", "; end;
		if (FT_VendorLocations[faction][npc][2]) then
			tex = a1.." ["..b1..c1..d1.."]";
		else
			tex = a1;
		end;
	end;
	return tex;
end;

function ERROR_ItemInfo(id,npc)
	local iN, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(id) or "";
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find any information on item ["..tostring(id).."] / '"..iN.."' :: Vendor:'"..npc.."' ? ".."Please report this on the website.", 60);
end;

function ERROR_VendorInformation(npc)
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find any information on '"..npc.."' ? ".."Please report this on the website.", 60);
end;

function ERROR_VendorLocation(npc)
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find location for '"..npc.."' ? ".."Please report this on the website.", 60);
end;

--function ERROR_VendorCoordinates(npc)
--	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."'"..npc.."' coordinates coming soon!", 60);
--end

function FT_FindVendorLocation(faction, npc)
	local vendorFound = FT_VendorLocations[faction][npc];
	if (vendorFound == nil) then
		vendorFound = FT_VendorLocations["Neutral"][npc];
		if (vendorFound) then
			faction = "Neutral";
		else
			return faction, nil;
		end;
	end;
	return faction, vendorFound;
end;

local function FT_CollectInfo(id)
	local itemInspect = FT_ItemInfo[id];
	local introCollected = FT.SOLD_BY
	local noteCollected, craftableCollected, worldeventCollected, professionCollected, dropCollected, mobsCollected;
	local colorCollected = FTUtils.FONT_COLOR;
	
	if (itemInspect.note ~= nil) then
		noteCollected = itemInspect.note;
	end;
--	if (itemInspect.we ~= nil) then
--		worldeventCollected = itemInspect.we;
--		colorCollected = FT.SPECIAL_VENDOR_COLOR;
--	end;
	if (itemInspect.drop ~= nil) then
		colorCollected = FT.DROP_COLOR;
		introCollected = FT.DROPPED_BY;
		dropCollected = true;
	end;
	if (itemInspect.quest ~= nil) or (itemInspect.fished ~= nil) then
		introCollected = FT.OPTAINABLE_BY;
	end;
	if (itemInspect.craft ~= nil) then
		colorCollected = FT.PATTERN_COLOR;
		introCollected = FT.MADE_BY;
--		noteCollected = string.format(FT.SKILL_LEVEL_REQUIRED, itemInspect.craft.sk);
		craftableCollected = true;
	end;
	if (itemInspect.mobs ~= nil) then
		introCollected = FT.DROPPED_BY;
		mobsCollected = true;
	end;
	if (itemInspect.useless ~= nil) then
		colorCollected = FT.USELESS_COLOR;
	end;
	if (itemInspect.ap ~= nil) then
		noteCollected = string.format(FT.AVERAGE_PRICE, FTUtils.GSC(itemInspect.ap));
	end;
	return introCollected, colorCollected, noteCollected, craftableCollected, worldeventCollected, professionCollected, dropCollected, mobsCollected;
end

function FT_OnTooltipSetItem(self)
	if ( not classicOnly ) then return; end;
	if (MerchantFrame:IsVisible()) then
		for buttonIndex = 1, MERCHANT_ITEMS_PER_PAGE do
			local button = _G["MerchantItem"..buttonIndex.."ItemButton"];
			if (self:IsOwned(button)) then return; end;
		end;
	end;

	local name, link = self:GetItem();
	if (not link) then return; end;
	-- workaround for OnTooltipSetItem being called twice for recipes
	local _, _, _, _, _, itemType, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
	if (itemType == FT.RECIPE and self.lastLink ~= link) then
		for i = 1, self:NumLines() do
			local line = _G[self:GetName().."TextLeft"..i];
			local text = line:GetText();
			if (text and string.sub(text, 1, 1) == "\n") then 
				self.lastLink = link;
				return; 
			end;
		end;
	end;
	self.lastLink = nil;
	
	if (link) then
		local _, _, itemID  = string.find(link, "item:(%d+)");
		itemID = tonumber(itemID);
--		itemID = 3914 --#TESTING#--
		
		local myFaction = UnitFactionGroup("player");
		local itemInfo = FT_VendorInfo[myFaction][itemID];
		
		if (not itemInfo) then
			itemInfo = FT_VendorInfo["Neutral"][itemID];
			if (not itemInfo) then
				itemInfo = false;
--				FT_NewItemInfo[myFaction] = itemID;
			end
		end
		
		if ( not FleecingTipDB.NoDisplay ) and ( itemInfo ) then
			
			local numVendors = 0;
			local soldBy = itemInfo;
			if ( itemInfo.vendors ) then
				soldBy = itemInfo.vendors;
				numVendors = table.getn(soldBy)
			else
				numVendors = table.getn(soldBy)
			end;
			if ( FT_VendorInfo[myFaction][itemID] ) and ( FT_VendorInfo["Neutral"][itemID] ) then
				soldBy = FTTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
				if ( FT_VendorInfo[myFaction][itemID].vendors ) and ( FT_VendorInfo["Neutral"][itemID].vendors ) then
					soldBy = FTTable.Merge(FT_VendorInfo[myFaction][itemID].vendors, FT_VendorInfo["Neutral"][itemID].vendors);
					numVendors = table.getn(soldBy)
				end;
			end;
			
			local numCost = 0;
			local costBy = false;
			if ( itemInfo.cost ) then
				costBy = itemInfo.cost;
				numCost = table.getn(costBy);
			end;
			if ( FT_VendorInfo[myFaction][itemID] ) and ( FT_VendorInfo["Neutral"][itemID] ) then
				if ( FT_VendorInfo[myFaction][itemID].cost ) and ( FT_VendorInfo["Neutral"][itemID].cost ) then
					costBy = FTTable.Merge(FT_VendorInfo[myFaction][itemID].cost, FT_VendorInfo["Neutral"][itemID].cost);
					numCost = table.getn(costBy);
				end;
			end;
			
--			if ( soldBy.cost ) then
--				costBy = soldBy.cost;
--				numCost = table.getn(costBy);
--			end;
			

			
			
--			if ( FT_VendorInfo[myFaction][itemID].vendors ) and ( FT_VendorInfo["Neutral"][itemID].vendors ) then
--				soldBy = FTTable.Merge(FT_VendorInfo[myFaction][itemID].vendors, FT_VendorInfo["Neutral"][itemID].vendors);
--			else
--				if ( FT_VendorInfo[myFaction][itemID].vendors ) then
--					soldBy = FT_VendorInfo[myFaction][itemID].vendors;
--				end;
--			end;
			
			
			
			
--			if ( FT_VendorInfo[myFaction][itemID].cost ) then
--				costBy = FT_VendorInfo[myFaction][itemID].cost;
--			end;
--			if ( FT_VendorInfo["Neutral"][itemID].cost ) then
--				costBy = FT_VendorInfo["Neutral"][itemID].cost;
--			end;
--			if ( FT_VendorInfo[myFaction][itemID].cost ) and ( FT_VendorInfo["Neutral"][itemID].cost ) then
--				costBy = FTTable.Merge(FT_VendorInfo[myFaction][itemID].cost, FT_VendorInfo["Neutral"][itemID].cost);
--			end;
			
			
			
--			print("myFaction = "..tostring(myFaction));
--			print("itemID = "..tostring(itemID));
			
--			print("numVendors = "..tostring(numVendors));
--			print("numCost = "..tostring(numCost));
			
			local intro, color, note, craftable, worldevent, profession, dropfound, mobDrops = FT_CollectInfo(itemID)
--			print("myFaction, intro, color, note, worldevent, profession, dropfound = "..tostring(myFaction)..", "..tostring(intro)..", "..tostring(color)..", "..tostring(note)..", "..tostring(worldevent)..", "..tostring(profession)..", "..tostring(dropfound));
			if (dropfound) then myFaction = "Drops"; end;
			
			if (numVendors > 1) then
				local num = 1;
				self:AddLine(intro..":", color.r, color.g, color.b);
				for i, aVendor in pairs(soldBy) do
					local locFaction, locVendor = FT_FindVendorLocation(myFaction,aVendor)
					
					if (locVendor ~= nil) or (profession and itemInfo.vendors) or (dropfound) then
						local vendorName = FT[aVendor] or aVendor;
						local vendorLocation = FT_GetVendorCoordinates(locFaction,aVendor);
						local vendorCost = "";
						if ( numCost > 1 ) and ( not FT_ItemInfo[itemID].ap ) then
							vendorCost = costBy[num];
							vendorCost = FT.COST_SPACE..FTUtils.GSC(vendorCost);
							num = num + 1;
						end;
						if (dropfound) then
							if (FT_VendorLocations["Drops"][vendorName][4] ~= nil) then
								local spec = FT_VendorLocations["Drops"][vendorName][4];
								vendorName = vendorName.." ("..spec..")";
							end;
						end;
						if (vendorLocation) ~= nil then
							self:AddLine(FT.INDENT..string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation)..vendorCost, color.r, color.g, color.b);
						else
							self:AddLine(FT.INDENT..vendorName..vendorCost, color.r, color.g, color.b);
--							if (not profession) then
--								ERROR_VendorCoordinates(aVendor);
--							end;
						end;
					else
						ERROR_VendorInformation(aVendor);
						if (aVendor == numVendors) then
							return false;
						end;
					end;
					
				end;
				
			elseif (numVendors == 1 and not itemInfo.boe) then
				
				local locFaction, locVendor = FT_FindVendorLocation(myFaction,soldBy[1]);
				if (locVendor ~= nil) or (profession and itemInfo.vendors) or (dropfound) then
					
					local vendorName = FT[soldBy[1]] or soldBy[1];
					local vendorLocation = FT_GetVendorCoordinates(locFaction,vendorName);
					local vendorCost = "";
					if ( numCost == 1 ) and ( not FT_ItemInfo[itemID].ap ) then
						vendorCost = costBy[1];
						vendorCost = FT.COST_SPACE..FTUtils.GSC(vendorCost);
					end;
					
					if (dropfound) then
						if (FT_VendorLocations["Drops"][vendorName][4] ~= nil) then
							local spec = FT_VendorLocations["Drops"][vendorName][4];
							vendorName = vendorName.." ("..spec..")";
						end;
					end;
					
					if (craftable) then
						vendorLocation = FT_ItemInfo[itemID].craft.prof or "";
						vendorLocation = vendorLocation.." "..string.format(FT.SKILL_LEVEL_REQUIRED, FT_ItemInfo[itemID].craft.sk);
					end;
					
					if (mobDrops) then
						vendorLocation = string.format(FT.LEVEL_DROP, FT_ItemInfo[itemID].mobs or "");
					end;
					
					if (vendorLocation ~= nil) and (vendorName ~= "Useless") then
						self:AddLine(intro.." "..string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation)..vendorCost, color.r, color.g, color.b);
					else
						if (vendorName == "Useless") then
							self:AddLine(FT.ITEM_WARNING, color.r, color.g, color.b);
						else
							self:AddLine(intro.." "..vendorName..vendorCost, color.r, color.g, color.b);
--							if (not profession) then
--								ERROR_VendorCoordinates(vendorName);
--							end;
						end;
					end;
					
				else
					if (not profession) then
						ERROR_VendorInformation(vendors[1]);
						return false;
					end;
				end;
				
			else
				
				local found = false;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						found = true;
						break; -- get out quick! :)
					end;
				end;
				if not (found) then
					local link, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID) or "";
					FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."'"..link.."'["..itemID.."] is listed but has no vendors.", 60);
					FTUtils.PrintOnce("Please report this on the mod website.", 60);
					return false;
				end;
				
			end;
			
			if (profession) then
				local color = FT.PATTERN_COLOR;
				
				local locFaction = myFaction;
				if (itemInfo.neutral) then locFaction = "Neutral"; end;
				
				local patternName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemInfo.id) or "";
				local vendorInfo = FT_VendorInfo[locFaction][itemInfo.id];
				local patternInfo = FT_ItemInfo[itemInfo.id];
				
				local iCost;
				if (patternInfo.i) then
					iCost = patternInfo.i; -- Different cost then gold!
				else
					if (patternInfo.b) then
						iCost = FTUtils.GSC(patternInfo.b); -- Gold cost here!w
					else
						iCost = UNKNOWN;
					end;
				end;
				
				local noNPC = table.getn(vendorInfo);
				self:AddLine(FT.MADE_BY.." "..itemInfo.pro, color.r, color.g, color.b);
				if (noNPC == 1) then
					local NPCname = vendorInfo[1];
					if (NPCname == "Trainer") then
						self:AddLine(string.format(FT.LEARN_AT, patternName), color.r, color.g, color.b);
					else
						local vendorLoc;
						locFaction, vendorLoc = FT_FindVendorLocation(locFaction,NPCname);
						if (vendorLoc ~= nil) then
							local NPCcoords = FT_GetVendorCoordinates(locFaction,NPCname);
							if (NPCcoords ~= nil) then
								self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost)..NPCname..", "..NPCcoords, color.r, color.g, color.b);
							else
								self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost)..NPCname, color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname);
							end;
						else
							ERROR_VendorInformation(NPCname);
						end;
					end;
				else
					self:AddLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost), color.r, color.g, color.b);
					for numName = 1, noNPC do
						local NPCname = vendorInfo[numName];
						locFaction, vendorLoc = FT_FindVendorLocation(locFaction,NPCname);
						if (vendorLoc ~= nil) then
							local NPCcoords = FT_GetVendorCoordinates(locFaction,NPCname);
							if (NPCcoords ~= nil) then
								self:AddLine(FT.INDENT..string.format(FT.VENDOR_LOCATION_FORMAT, NPCname, NPCcoords), color.r, color.g, color.b);
							else
								self:AddLine(FT.INDENT..NPCname, color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname);
							end;
						else
							ERROR_VendorInformation(NPCname)
						end;
					end;
				end;
			end;
			
--			if ( FT_ItemInfo[itemID].ap ) then
--				local vendorCost = FT_ItemInfo[itemID].ap;
--				vendorCost = FTUtils.GSC(vendorCost);
--				self:AddLine(FT.INDENT..format(FT.PRICE, vendorCost), color.r, color.g, color.b);
--			end;
			
			if (note ~= nil) then
				self:AddLine(note, color.r, color.g, color.b);
			end;
--			if (worldevent ~= nil) then
--				self:AddLine(FT.WORLD_EVENT..": "..worldevent.." ("..CheckEventActive(worldevent)..")", color.r, color.g, color.b);
--			end;
			
			FTUtils.PlaySound(itemID, 30);
			return true;
		end;
		
		-- FactionFriend also shows tooltips for this stuff; if it's present, let it take over
--		if (FFF_Config and not FFF_Config.NoTooltip) then return; end;
	
	end;
	
end;

function FT_OnEvent(self, event, arg1)

-- In Mists of Pandaria, the game freezes if you attempt to call GetMerchantNumItems() when MERCHANT_SHOW or MERCHANT_UPDATE is called.
-- To correct this, a delay is instituted.  I found that even with .01 delay, it still doesn't freeze.  I can live with a .01 delay.
-- I do worry about the impact to performance this code might have, though.  If you know a better method, feel free to let me know.

	local delayFrame = CreateFrame("FRAME");
	local delayTotal = 0;
	function delayFrame:OnUpdate(self, elapsed)
		delayTotal = delayTotal + self;
		if delayTotal >= .01 then
			delayTotal = 0;
			delayFrame:SetScript("OnUpdate", nil);
			FT_ScanMerchant();
		end;
	end;

	if( event == "MERCHANT_SHOW" or event == "MERCHANT_UPDATE" ) then
--		FT_ScanMerchant();		-- This is where the function was called before delay was instituted.
		delayFrame:SetScript("OnUpdate", delayFrame.OnUpdate);
	end;
	
end;

function FT_OnLoad(self)

	-- Register Slash Commands
	SLASH_FLEECINGTIP1 = "/fleecingtip";
	SLASH_FLEECINGTIP2 = "/ft";
	SlashCmdList["FLEECINGTIP"] = function(msg)
--		FT_ChatCommandHandler(msg);
		FT_DisabledChatCommandHandler(msg);
	end;

	-- Register for Events
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_UPDATE");
	
	FT_HookTooltip(GameTooltip);
	FT_HookTooltip(ItemRefTooltip);
		
end;

function FT_ChatCommandHandler(msg)
	
	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		FTUtils.Print(addon.." "..version..":");
		FTUtils.Print("/fleecingtip (or /ft)");
		FTUtils.Print("- "..FTUtils.Hilite("help").." - Display this help list.");
		FTUtils.Print("- "..FTUtils.Hilite("[item link]").." - Show info for an item in the chat window.");
		FTUtils.Print("- "..FTUtils.Hilite("status").." - Check current settings.");
		FTUtils.Print("- "..FTUtils.Hilite("money icon").." | "..FTUtils.Hilite("short").." | "..FTUtils.Hilite("long").." - Money format display.");
		FTUtils.Print("- "..FTUtils.Hilite("sound on").." | "..FTUtils.Hilite("off").." - Found an item sound.");
		FTUtils.Print("- "..FTUtils.Hilite("show on").." | "..FTUtils.Hilite("off").." - Display 'Fleecing Tip' information in tooltip.");
		return;
	end
	
	if (msg == "version") then
		FTUtils.Print(addon.." "..version);
		return;
	end
	
	if ( msg == "sound on" ) or ( msg == "sound off" ) then
		FleecingTipDB.NoSound = not FleecingTipDB.NoSound
		if (FleecingTipDB.NoSound) then
			FTUtils.Print(addon.." sound is now set to "..FTUtils.Hilite("off")..".");
		else
			FTUtils.Print(addon.." sound is now set to "..FTUtils.Hilite("on")..".");
		end
		return;
	end
	
	if ( msg == "money icon" ) then
		FleecingTipDB.CoinFormat = "icon";
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("icon")..".");
		return;
	end
	
	if ( msg == "money short" ) then
		FleecingTipDB.CoinFormat = "short";
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("short")..".");
		return;
	end
	
	if ( msg == "money long" ) then
		FleecingTipDB.CoinFormat = "long";
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("long")..".");
		return;
	end
	
	if ( msg == "show on" ) then
		FleecingTipDB.NoDisplay = false
		FTUtils.Print(addon.." is "..FTUtils.Hilite("on")..", and showing tooltip information.");
		return;
	end
	
	if ( msg == "show off" ) then
		FleecingTipDB.NoDisplay = true
		FTUtils.Print(addon.." showing tooltip information is "..FTUtils.Hilite("off")..".");
		return;
	end
	
	if ( msg == "status" ) then
		FTUtils.Print(addon.." status:");
		FTUtils.Print("Money format is "..FTUtils.Hilite(FleecingTipDB.CoinFormat)..".");
		if (FleecingTipDB.NoSound) then
			FTUtils.Print("Sound is "..FTUtils.Hilite("off")..".");
		else
			FTUtils.Print("Sound is "..FTUtils.Hilite("on")..".");
		end
		local gotSomething;
		if (not FleecingTipDB.NoDisplay) then
			FTUtils.Print("Show "..FTUtils.Hilite("on").." Displaying tooltip information for "..addon..".");
			gotSomething = true;
		end
		if (not gotSomething) then
			FTUtils.Print("Show "..FTUtils.Hilite("off ")..addon.." is not adding any information to tooltip.");
		end
		return;
	end
	
	if (msg == "test") then
		FTUtils.Print(addon..", Data Stats:");
		local itemInfoCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			local found = false;
			for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
				if (FT_VendorInfo[faction][itemID]) then
					found = true;
				end
			end
			if not (found) then
				FTUtils.Print("Item ID "..itemID.." not found in FT_VendorInfo.");
			end
			itemInfoCount = itemInfoCount + 1;
		end
		FTUtils.Print("  - "..itemInfoCount.." entries in FT_ItemInfo.");
		for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
			local vendorInfoCount = 0;
			for itemID in pairs(FT_VendorInfo[faction]) do
				if (FT_ItemInfo[itemID] == nil) then
					FTUtils.Print("Item ID "..itemID.." not found in FT_ItemInfo.");
				end
				vendorInfoCount = vendorInfoCount + 1;
			end
			FTUtils.Print("  - "..vendorInfoCount.." entries in FT_VendorInfo["..faction.."].");
		end
		local dropCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			if (FT_ItemInfo[itemID].i == "Drop") then
				local dropfound = false;
				local dropName = tostring(_G.UNKNOWN);
				local loc = dropName;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						dropName = FT_VendorInfo[faction][itemID][1]
						loc = faction
--						print("Name: "..tostring(dropName)); -- *Debug*
						if ( FT_VendorLocations["Drops"][dropName]) then
							dropfound = true;
						end
					end
				end
				if not (dropfound) then
					FTUtils.Print("Drop '"..dropName.."' not found for ["..loc.."] "..itemID.." itemID.");
				end
				dropCount = dropCount + 1;
			end
		end
		FTUtils.Print("  - "..dropCount.." Drop entries.");
		return;
	end

	local _, _, cmd, args = string.find(msg, "^([%l%d']+) *(.*)");
	if (cmd) then cmd = string.lower(cmd); end
		
	if (args == nil or args == "") then
		args = msg;
	end
	local postedText;
	for itemLink in string.gmatch(args, "|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r") do
		postedText = nil;
		local _, _, itemID  = string.find(itemLink, "item:(%d+)");
		if (itemID == nil or itemID == "") then
			FTUtils.Print("Usage: "..FTUtils.Hilite("/ft info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		FTUtils.Print(addon..": found...");
		local itemInfo = FT_ItemInfo[itemID];
		if (itemInfo) then
			local myFaction = UnitFactionGroup("player");
			local vendors = FTTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
			if (FT_NewVendorInfo[myFaction] and FT_NewVendorInfo[myFaction][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo[myFaction][itemID]);
			end
			if (FT_NewVendorInfo["Neutral"] and FT_NewVendorInfo["Neutral"][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo["Neutral"][itemID]);
			end
			local note = itemInfo.note;
			local intro = itemLink..": "..string.format(FT.SOLD_BY, FTUtils.GSC(itemInfo.b));

			if (vendors == nil or vendors == {}) then
				FTUtils.Print(FTUtils.Red(addon.." "..version.." error: ")..itemLink.."("..itemID..") is listed but has no vendors. Please report this at the website you downloaded from.");
				return;
			end
			FTUtils.Print(intro);
			for i, aVendor in pairs(vendors) do
				local vendorName = FT[aVendor] or aVendor;
--				local vendorLocation = FT[FT_VendorLocations[myFaction][aVendor]] or FT_VendorLocations[myFaction][aVendor];
--				if (not vendorLocation) then
--					vendorLocation = FT[FT_NewVendorLocation[myFaction][aVendor]] or FT_NewVendorLocation[myFaction][aVendor];
--				end
--				local vendorNPC = FT_GetVendorCoordinates(vendorName)
				local vendorNPC = FT_GetVendorCoordinates(myFaction,aVendor)
--				if (vendorNPC) ~= nil then
					FTUtils.Print(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorNPC));
--				else
--					FTUtils.Print(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation));
--				end;
			end
			if (note and note ~= "") then
				FTUtils.Print(FTUtils.Hilite(note));
			end
			postedText = 1;
		end
		
		if (not postedText) then
			FTUtils.Print("Nothing known about "..itemLink..".");
		end
		return;
	end
	if (postedText) then
		return;
	end
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FT_ChatCommandHandler("help");

end;

function FT_DisabledChatCommandHandler(msg)
	if ( msg == "" ) then
		FTUtils.Print(FTUtils.Gray(addon.." "..version.." Commands are disabled. (Sorry!)"));
	end;
end;

function FT_CheckMerchant(itemID)
	for merchantIndex = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(merchantIndex);
		local _, _, merchantItemID  = string.find(link, "item:(%d+)");
		if (tonumber(merchantItemID) == itemID) then
			return true;
		end
	end
	return false;
end

function FT_ScanMerchant()
	for index = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(index);
		if ( link ) then
			local _, _, _, _, _, itemType, subType, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
			if (not FT_ItemIsBoP(index)) and (not FT_ItemBlacklist(link)) and ((itemType == FT.RECIPE) or (itemType == FT.MISCELLANEOUS and ((subType == FT.PET) or (subType == FT.MOUNT))) or (itemType == FT.CONTAINER and subType == FT.BAG)) then
				local itemID = FTUtils.DecomposeItemLink(link);
				local added;
				local _, _, price, _, _, _, _, extendedCost = GetMerchantItemInfo(index);
				local priceSummmary;
				if (price and price > 0) then
				 	priceSummmary = FTUtils.GSC(price);
				else
					priceSummmary = "";
				end
				local itemSummary = "";
				if (price == 0 and extendedCost) then
					itemCount = GetMerchantItemCostInfo(index);
					if (itemCount and itemCount > 0) then
						for costIndex = 1, itemCount, 1 do
							local itemTexture, itemValue, itemLink, itemName = GetMerchantItemCostItem(index, costIndex);
							if (not itemName) then
								itemName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemLink);
								if (not itemName) then
									itemName = "Unknown Item";
								end
							end
							itemSummary = itemSummary .. string.format(" + %d %s", itemValue, itemName);
						end
					end
					priceSummmary = priceSummmary .. itemSummary;
					itemSummary = string.gsub(itemSummary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, " %+ $", "");
					if (itemSummary == "") then
						itemSummary = nil;
					end
				end
				if (not FT_ItemInfo[itemID]) then
--					print("|cffff9900**FT ERROR** link:"..tostring(link).." itemID="..tostring(itemID).." price:"..tostring(price).." info:"..tostring(itemSummary).."|r")
					FT_NewItemInfo[itemID] = { b=price, i=itemSummary, note="" };
					added = true;
				end
				local vendorName = UnitName("npc");
				local vendorFaction = UnitFactionGroup("npc");
				if (vendorFaction ~= UnitFactionGroup("player")) then
					vendorFaction = "Neutral";
				end
				if (FT_VendorInfo[vendorFaction][itemID] == nil or FTTable.KeyOf(FT_VendorInfo[vendorFaction][itemID], vendorName) == nil) then
					FT_NewVendorInfo[vendorFaction] = FT_NewVendorInfo[vendorFaction] or {};
					FT_NewVendorInfo[vendorFaction][itemID] = FT_NewVendorInfo[vendorFaction][itemID] or {};
					if (FTTable.KeyOf(FT_NewVendorInfo[vendorFaction][itemID], vendorName) == nil) then
						table.insert(FT_NewVendorInfo[vendorFaction][itemID], vendorName);
						added = true;
					end
				end
				FT_NewVendorLocation[vendorFaction] = FT_NewVendorLocation[vendorFaction] or {};
				FT_NewVendorLocation[vendorFaction][vendorName] = FT_NewVendorLocation[vendorFaction][vendorName] or {};
				if (not FT_VendorLocations[vendorFaction][vendorName] and not FT_NewVendorLocation[vendorFaction][vendorName]) then
					local vendorZone = GetRealZoneText();
					FT_NewVendorLocation[vendorFaction][vendorName] = vendorZone;
					added = true;
				end
				if (added) then
					FTUtils.PrintOnce(string.format(addon.." %s has detected new data!  Please report the following message:  %s [%s] sold for %s by %s (%s) in %s.", version, link, itemID, FTUtils.Hilite(priceSummmary), FTUtils.Hilite(vendorName), vendorFaction, FTUtils.Hilite(GetRealZoneText())));
				end
			end
		end
	end
end

-- Attempt #2 at a BoP check, seems to work better =P
-- Version 4.2b adds a BoA check as well.
function FT_ItemIsBoP(index)
	FT_BoPCheckingTooltip:ClearLines();
	FT_BoPCheckingTooltip:SetMerchantItem(index);
	for lineNum=1, FT_BoPCheckingTooltip:NumLines() do
		if lineNum > 4 then
			return false;
		end
		local leftText = _G["FT_BoPCheckingTooltipTextLeft"..lineNum]:GetText();
		if string.find(leftText, ITEM_BIND_ON_PICKUP) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_BNETACCOUNT) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_ACCOUNT) then
			return true;
		end
	end
end

-- A list of items that we don't want to detect new locations for.
function FT_ItemBlacklist(link)
	local result = false
	local FT_Blacklist = {
	[4496] = true,		-- Small Brown Pouch, sold everywhere.
	[4497] = true,		-- Heavy Brown Bag, sold everywhere.
	[4498] = true,		-- Brown Leather Satchel, sold everywhere.
	[4499] = true,		-- Huge Brown Sack, sold everywhere.
--	[17200] = true,		-- Recipe: Gingerbread Cookie.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
--	[17201] = true,		-- Recipe: Egg Nog.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
	[20752] = true,		-- Formula: Minor Mana Oil.  Blacklisted for having a lot of vendors.
	[20753] = true,		-- Formula: Lesser Wizard Oil.  Blacklisted for having a lot of vendors.
	[20758] = true,		-- Formula: Minor Wizard Oil.  Blacklisted for having a lot of vendors.
	[21099] = true,		-- Recipe: Smoked Sagefish.  Blacklisted for having a lot of vendors.
	[21219] = true,		-- Recipe: Sagefish Delight.  Blacklisted for having a lot of vendors.
	[22200] = true,		-- Silver Shafted Arrow.  Blacklisted because it is not a pet but is detected as one.
	[22307] = true,		-- Pattern: Enchanted Mageweave Pouch.  Blacklisted for having a lot of vendors.
	[30744] = true,		-- Draenic Leather Pack, sold everywhere.
	[60335] = true,		-- Thick Hide Pack, sold everywhere.
--	[74610] = true,		-- Lunar Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Alliance.
--	[74611] = true,		-- Festival Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Horde.
	};
	local itemID = FTUtils.DecomposeItemLink(link);
	if (FT_Blacklist[itemID]) then
		result = true;
	end;
	return result
end;
