------------------------------------------------
-- FTUtils.lua
-- Useful utilities
-- Some original work from GFWUtils many thanks!
------------------------------------------------

local HIGHLIGHT_FONT_COLOR_CODE = _G.HIGHLIGHT_FONT_COLOR_CODE;
local RED_FONT_COLOR_CODE = _G.RED_FONT_COLOR_CODE;
local GRAY_FONT_COLOR_CODE = _G.GRAY_FONT_COLOR_CODE;
local LIGHTYELLOW_FONT_COLOR_CODE = _G.LIGHTYELLOW_FONT_COLOR_CODE;
local FONT_COLOR_CODE_CLOSE = _G.FONT_COLOR_CODE_CLOSE;

local UIERRORS_HOLD_TIME = _G.UIERRORS_HOLD_TIME;

local GOLD_AMOUNT_TEXTURE_STRING = _G.GOLD_AMOUNT_TEXTURE_STRING;
local SILVER_AMOUNT_TEXTURE = _G.SILVER_AMOUNT_TEXTURE;
local COPPER_AMOUNT_TEXTURE = _G.COPPER_AMOUNT_TEXTURE;

local GOLD_AMOUNT = _G.GOLD_AMOUNT;
local SILVER_AMOUNT = _G.SILVER_AMOUNT;
local COPPER_AMOUNT = _G.COPPER_AMOUNT;

local GOLD_AMOUNT_SYMBOL = _G.GOLD_AMOUNT_SYMBOL;
local SILVER_AMOUNT_SYMBOL = _G.SILVER_AMOUNT_SYMBOL;
local COPPER_AMOUNT_SYMBOL = _G.COPPER_AMOUNT_SYMBOL;

------------------------------------------------

FTUtils = {};

FTUtils.FONT_COLOR = {r=0.25, g=1.0, b=1.0};

-- Shortcuts for common text color codes
FTUtils.Hilite = function(text)
	if (text == nil) then return nil; end
	return HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
FTUtils.Red = function(text)
	if (text == nil) then return nil; end
	return RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
FTUtils.Gray = function(text)
	if (text == nil) then return nil; end
	return GRAY_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end
FTUtils.LightYellow = function(text)
	if (text == nil) then return nil; end
	return LIGHTYELLOW_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
end

-- Plays sound only on a new item
FTUtils.PlaySound = function(itemNum, interval)
	local noSound = false;
	local FT_soundBlacklist = {
	 -- itemID's that don't need sound...
		[5576] = true, 
		[5575] = true, 
		[5574] = true, 
		[5573] = true, 
		[5572] = true, 
		[5571] = true, 
		[2657] = true, 
		[1725] = true, 
		[1685] = true, 
		[857] = true, 
		[856] = true, 
		[828] = true, 
		[805] = true, 
		[804] = true,
	};
	if (FT_soundBlacklist[itemNum]) then
		noSound = true;
	end;
	if (FleecingTipDB.NoSound) or (noSound) then return; end;
	local Act = nil;
	if (FTUtils.PlaySoundCache == nil) then
		FTUtils.PlaySoundCache = itemNum;
		Act = true;
	end
	if (FTUtils.PlaySoundCache ~= itemNum) then
		FTUtils.PlaySoundCache = itemNum;
		Act = true;
	end
	if (Act) then
		PlaySound(3580); -- sheep death
	end
end

-- Prints a message to the chat frame
FTUtils.Print = function(message, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage(message, (r or FTUtils.FONT_COLOR.r), (g or FTUtils.FONT_COLOR.g), (b or FTUtils.FONT_COLOR.b));
end

-- Prints a message to the chat frame once per interval (seconds) or once per session if interval is nil
FTUtils.PrintOnce = function(message, interval, r, g, b)
	if (FTUtils.PrintOnceCache == nil) then
		FTUtils.PrintOnceCache = {};
	end
	if (interval == nil and GFWUtils.PrintOnceCache[message]) then
		return;
	end
	if (interval and FTUtils.PrintOnceCache[message] and GetTime() - FTUtils.PrintOnceCache[message] < interval) then
		return;
	end
	DEFAULT_CHAT_FRAME:AddMessage(message, (r or FTUtils.FONT_COLOR.r), (g or FTUtils.FONT_COLOR.g), (b or FTUtils.FONT_COLOR.b));
	FTUtils.PrintOnceCache[message] = GetTime();
end

-- large number break down # NOW USE Blizzards own BreakUpLargeNumbers #
--local function breakdownNumbers(amount)
--	amount = 1234567890 -- for testing purposes!!!!
--	assert (type (amount) == "number" or type (amount) == "string")
--	local result = ""
--	local sign, before, after = string.match (tostring (amount), "^([%+%-]?)(%d*)(%.?.*)$")
--	while string.len (before) > 3 do
--		result = ","..string.sub (before, -3, -1)..result
--		before = string.sub (before, 1, -4)
--	end
--	return sign..before..result..after
--end

-- Formats money text for gold, silver, copper
-- complete redone method for icon, long & short display
FTUtils.GSC = function(money)
	if (not money) then return 0; end;
	local gold, silver, copper = math.floor(money/(100*100)), math.abs(mod(money/100,100)), math.abs(mod(money,100));
	local size = 12;
	
	local gsc = "|cffffffff"..string.format(GOLD_AMOUNT_TEXTURE_STRING, BreakUpLargeNumbers(gold), size, size).."|r";
	if silver > 0 then gsc = gsc.." |cffffffff"..string.format(SILVER_AMOUNT_TEXTURE, silver, size, size).."|r"; end;
	if copper > 0 then gsc = gsc.." |cffffffff"..string.format(COPPER_AMOUNT_TEXTURE, copper, size, size).."|r"; end;
	
	if FleecingTipDB.CoinFormat == "long" then 
		gsc = "|cffffd700"..string.format(GOLD_AMOUNT, BreakUpLargeNumbers(gold)).."|r,";
		if silver > 0 then gsc = gsc.." |cffc0c0c0"..string.format(SILVER_AMOUNT, silver).."|r,"; end;
		if copper > 0 then gsc = gsc.." |cffc87533"..string.format(COPPER_AMOUNT, copper).."|r,"; end;
	end;
	
	if FleecingTipDB.CoinFormat == "short" then
		gsc = string.format("|cffffffff%d|cffffd700%s|r", BreakUpLargeNumbers(gold), GOLD_AMOUNT_SYMBOL)
		if silver > 0 then gsc = gsc..string.format(" |cffffffff%d|cffc0c0c0%s|r", silver, SILVER_AMOUNT_SYMBOL); end;
		if copper > 0 then gsc = gsc..string.format(" |cffffffff%d|cffc87533%s|r", copper, COPPER_AMOUNT_SYMBOL); end;
	end;
	
	if money < 1e4 then -- < 10000
        gsc = "|cffffffff"..string.format(SILVER_AMOUNT_TEXTURE, silver, size, size).."|r";
		if copper > 0 then gsc = gsc.." |cffffffff"..string.format(COPPER_AMOUNT_TEXTURE, copper, size, size).."|r"; end;
		if FleecingTipDB.CoinFormat == "long" then
			gsc = "|cffc0c0c0"..string.format(SILVER_AMOUNT, silver).."|r,";
			if copper > 0 then gsc = gsc.." |cffc87533"..string.format(COPPER_AMOUNT, copper).."|r,"; end;
		end;
		if FleecingTipDB.CoinFormat == "short" then
			gsc = string.format("|cffffffff%d|cffc0c0c0%s|r", silver, SILVER_AMOUNT_SYMBOL);
			if copper > 0 then gsc = gsc..string.format(" |cffffffff%d|cffc87533%s|r", copper, COPPER_AMOUNT_SYMBOL); end;
		end;
	end;
	
    if money < 1e2 then -- < 100
        gsc = "|cffffffff"..string.format(COPPER_AMOUNT_TEXTURE, copper, size, size).."|r";
		if FleecingTipDB.CoinFormat == "long" then gsc = "|cffc87533"..string.format(COPPER_AMOUNT, copper).."|r,"; end;
		if FleecingTipDB.CoinFormat == "short" then gsc = string.format("|cffffffff%d|cffc87533%s|r", copper, COPPER_AMOUNT_SYMBOL); end;
    end;
	
	return gsc;
end

FTUtils.ItemLink = function(linkInfo)
--	local sName, sLink, iQuality, iLevel, _, sType, sSubType, iCount, _, _, _, _, _, _, _, _, _ = GetItemInfo(linkInfo);
	local sLink = select(2, GetItemInfo(linkInfo));
	return sLink;
end

FTUtils.DecomposeItemLink = function(link)
	local _, _, color, itype, itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID, linkLvl, reforging, name = string.find(link, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	itemID = tonumber(itemID);
	enchant = tonumber(enchant);
	gem1 = tonumber(gem1);
	gem2 = tonumber(gem2);
	gem3 = tonumber(gem3);
	gem4 = tonumber(gem4);
	suffix = tonumber(suffix);
	uniqueID = tonumber(uniqueID);
	return itemID, enchant, gem1, gem2, gem3, gem4, suffix, uniqueID;
end
